#include "NebulaDevice.h"
#include "IOTCDevice.h"
#include "RDTServer.h"
#include "disposable_params.h"
#include "common.h"
#include <ctype.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static unsigned int gBindAbort = 0;
static char gUdid[MAX_UDID_LENGTH + 1] = {0};
static char gPinCode[MAX_PIN_CODE_LENGTH + 1] = {0};
static char gSecretId[MAX_NEBULA_SECRETID_LENGTH + 1] = {0};
static char gPsk[MAX_NEBULA_PSK_LENGTH + 1] = {0};

int get_profile_from_path(const char *path, TUTKJsonObject **profile_obj) {
    int ret = 0;
    FILE *fp = NULL;
    char *profile = NULL;
    fp = fopen(path, "r");
    if (fp == NULL) {
        printf("Invalid path [%s] of profile\n", path);
        return -1;
    }

    fseek(fp, 0, SEEK_END);
    long fsize = ftell(fp);
    fseek(fp, 0, SEEK_SET);

    profile = malloc(fsize + 1);
    fread(profile, 1, fsize, fp);
    fclose(fp);
    profile[fsize] = '\0';

    ret = TUTK_Json_Obj_Create_From_String(profile, profile_obj);
    free(profile);
    if (ret != NEBULA_ER_NoERROR) {
        printf("Invalid profile\n");
        return -1;
    }
    return 0;
}

void identity_handle(NebulaDeviceCtx *device, const char *identity, char *psk, unsigned int psk_size) {
    printf("[%s]%s\n", __func__, identity);
    if (psk_size > strlen(gPsk))
        strcpy(psk, gPsk);
}

int command_handle(NebulaDeviceCtx *device, const char *identity, const char *func, const TUTKJsonObject *json_args, TUTKJsonObject **json_response) {
    printf("[%s] %s\n", __func__, func);
    return 400;
}

int settings_change_handle(NebulaDeviceCtx *device, const char *settings) {
    printf("[%s]settings[%s]\n", __func__, settings);
    return 0;
}

int device_login_state_handle(NebulaDeviceCtx *device, NebulaDeviceLoginState state) {
    printf("[%s]login state[%d]\n", __func__, state);
    return 0;
}

int main(int argc, char *argv[]) {
    int ret = 0;
    int sid = -1;
    int rdt_id = 0;
    TUTKJsonObject *profile_obj = NULL;
    const char *profile = NULL;
    const char *license_key = NULL;
    NebulaDeviceCtx *device_ctx = NULL;
    char buf[1024] = {0};
    int data_len = 0;

    if (argc != 3) {
        printf("%s [UDID] [Profile]\n", argv[0]);
        return -1;
    }

    strcpy(gUdid, argv[1]);

    if (get_profile_from_path(argv[2], &profile_obj) < 0) {
        return -1;
    }

    ret = loadDisposableParams(gPinCode, gSecretId, gPsk);
    if (ret < 0) {
        printf("loadDisposableParams()=%d\n", ret);
        return -1;
    }

    license_key = getenv(ENV_LICENSE_KEY);
    if (license_key == NULL) {
        printf("Environment variable %s must be set!!\n", ENV_LICENSE_KEY);
        return -1;
    }

    //Set log file path
    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));

    log_attr.path = "./log_iotc_device.txt";
    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_rdt_device.txt";
    ret = RDT_Set_Log_Attr(log_attr);
    printf("RDT_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_nebula_device.txt";
    ret = Nebula_Set_Log_Attr(log_attr);
    printf("Nebula_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_global_device.txt";
    ret = TUTK_Set_Log_Attr(log_attr);
    printf("TUTK_Set_Log_Attr ret[%d]\n", ret);

    ret = TUTK_SDK_Set_License_Key(license_key);
    printf("TUTK_SDK_Set_License_Key() ret[%d]\n", ret);
    if (ret < 0) {
        return -1;
    }

    ret = Nebula_Initialize();
    printf("Nebula_Initialize() ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    ret = IOTC_Initialize2(0);
    printf("IOTC_Initialize() ret[%d]\n", ret);
    if (ret != IOTC_ER_NoERROR) {
        return -1;
    }

    ret = RDT_Initialize();
    printf("RDT_Initialize() rdt_channel[%d]\n", ret);
    if (ret < 0) {
        return -1;
    }

    profile = TUTK_Json_Obj_To_String(profile_obj);
    ret = Nebula_Device_New(gUdid, gSecretId, profile, command_handle, identity_handle, settings_change_handle, &device_ctx);
    printf("Nebula_Device_New() ret[%d]\n", ret);
    TUTK_Json_Obj_Release(profile_obj);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    ret = Nebula_Device_Login(device_ctx, device_login_state_handle);
    printf("Nebula_Device_Login() ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        return -1;
    }

    while (1) {
        ret = Nebula_Device_Bind(device_ctx, gPinCode, gPsk, 60000, &gBindAbort);
        printf("Nebula_Device_Bind() ret[%d]\n", ret);
        if (ret == NEBULA_ER_NoERROR)
            break;
        else if (ret != NEBULA_ER_TIMEOUT) {
            return -1;
        }
    }

    ret = IOTC_Device_Login_By_Nebula(device_ctx);
    printf("IOTC_Device_Login_By_Nebula() ret = %d\n", ret);
    if (ret < 0) {
        return -1;
    }

    while (1) {
        sid = IOTC_Device_Listen_By_Nebula(device_ctx, 3000);
        printf("IOTC_Device_Listen_By_Nebula() ret[%d]\n", sid);
        if (sid == IOTC_ER_TIMEOUT) {
            continue;
        } else {
            break;
        }
    }

    if (sid < 0) {
        return -1;
    }

    rdt_id = RDT_Server_Create(sid, 0, NULL, NULL, 3000);
    printf("RDT_Server_Create() rdt_id[%d]\n", rdt_id);
    if (ret < 0) {
        return -1;
    }

    while(1){
        data_len = RDT_Read(rdt_id, buf, sizeof(buf), 100000);
        printf("RDT_Read() ret[%d]\n", data_len);
        if (data_len == RDT_ER_TIMEOUT) {
            continue;
        } else {
            break;
        }
    }

    if (data_len >= 0) {
        printf("RDT receive [%s]\n", buf);
        for (int i = 0; i < data_len; i++) {
            if (buf[i] == '\0') {
                break;
            }
            buf[i] -= 0x20; // small letter to capital letter for ASCII
        }

        ret = RDT_Write(rdt_id, buf, data_len);
        if (ret < 0) {
            printf("RDT_Write() ret[%d]\n", ret);
        }
    }

    RDT_Destroy(rdt_id);
    RDT_DeInitialize();

    IOTC_Session_Close(sid);
    IOTC_DeInitialize();

    Nebula_Device_Delete(device_ctx);
    Nebula_DeInitialize();
    return 0;
}
